/**
 ** rmp440CntrlTaskCodels.c
 **
 ** Codels used by the control task rmp440CntrlTask
 **
 ** Author:
 ** Date:
 **
 **/

#include <math.h>
#include <stdio.h>
#include <stdint.h>

#include <portLib.h>
#include <rmp440/rmp440.h>

#include "rmp440Const.h"
#include "server/rmp440Header.h"

/*------------------------------------------------------------------------
 * controlCmdParam  -  control codel of CONTROL request SetCmdParam
 *
 * Description:
 * Report: OK
 *                 S_rmp440_BAD_CMD_PARAMETERS
 *
 * Returns:    OK or ERROR
 */

STATUS
controlCmdParam(GENPOS_CART_CMD_PARAM *cmd, int *report)
{
	/* ... add your code here ... */
	return OK;
}

/*------------------------------------------------------------------------
 * computeStopRef  -  control codel of CONTROL request Stop
 *
 * Description:
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
computeStopRef(int *report)
{
	GENPOS_CART_STATE *robotId;           /* Config robot */
	GENPOS_CART_REF *refId;               /* Consigne */
	double dtArret, dtRetard;  /* Temps prise en compte et temps deceleration */
	double dSArret, dTArret;   /* Longueurs lin et ang de decel */
	double angleFinal;         /* Angle a l'arrivee */
	double period;

	robotId = &rmp440DataStrId->robot;
	refId = &rmp440DataStrId->ref;

	/* Longueur d'arret */
	period = EXEC_TASK_PERIOD(RMP440_MOTIONTASK_NUM);
	dtArret = MAX (fabs(robotId->v) / RMP_DEFAULT_MAXIMUM_DECEL,
		       fabs(robotId->w) / RMP_DEFAULT_MAXIMUM_YAW_ACCEL);
	dtRetard =  1 * period;
	dSArret = robotId->v * (dtArret / 2 + dtRetard);
	dTArret = robotId->w * (dtArret / 2 + dtRetard);
	angleFinal = robotId->theta + dTArret;

	/* Consigne */
	refId->x = robotId->xRob + dSArret * cos (angleFinal);
	refId->y = robotId->yRob + dSArret * sin (angleFinal);
	refId->v = 0. ;
	refId->w = 0. ;
	return(OK);
}

/*------------------------------------------------------------------------
 * joystickCtrl  -  control codel of EXEC request JoystickOn
 *
 * Description:    
 * Report: OK
 *                 S_rmp440_BAD_REF
 *                 S_rmp440_RMP_LIB_ERROR
 *                 S_rmp440_JOYSTICK_ERROR
 *                 S_rmp440_MOTORS_OFF
 *
 * Returns:    OK or ERROR
 */

STATUS
joystickCtrl(int *report)
{
	RMP440_STATUS_STR *status = &SDI_F->status;
	int retval = OK;

	/* Check if mode changed */
	switch (status->rs_mode) {
	case RMP440_POWER_CORD:
		*report = S_rmp440_POWER_CORD_CONNECTED;
		return ERROR;
	case RMP440_EMERGENCY:
		*report = S_rmp440_EMERGENCY_STOP;
		return ERROR;
	case RMP440_MOTORS_OFF:
		*report = S_rmp440_MOTORS_OFF;
		return ERROR;
	}
	return OK;
}

/*------------------------------------------------------------------------
 * trackStart  -  control codel of EXEC request Track
 *
 * Description:    Check tracking mode compatibility and start tracking
 *                 if possible
 * Report: OK
 *                 S_rmp440_POSTER_NOT_FOUND
 *                 S_rmp440_BAD_REF
 *                 S_rmp440_CMD_STOP_TRACK
 *                 S_rmp440_MOTORS_OFF
 *
 * Returns:    OK or ERROR
 */

STATUS
trackStart(RMP440_TRACK_PARAMS *params, int *report)
{
	RMP440_STATUS_STR *status = &SDI_F->status;
	int retval = OK;

	/* Check if mode changed */
	switch (status->rs_mode) {
	case RMP440_POWER_CORD:
		*report = S_rmp440_POWER_CORD_CONNECTED;
		return ERROR;
	case RMP440_EMERGENCY:
		*report = S_rmp440_EMERGENCY_STOP;
		return ERROR;
	case RMP440_MOTORS_OFF:
		*report = S_rmp440_MOTORS_OFF;
		return ERROR;
	}
	/* Start correct tracking mode */
	*report = 0;
	switch (params->mode) {
	case GENPOS_NO_TRACKING:
		status->rs_mode = RMP440_IDLE;
		rmp440DataStrId->trackStr.trackMode = params->mode;
		*report = S_rmp440_CMD_STOP_TRACK;
		retval = ERROR;
		break;
	case GENPOS_TRACK_POS:
	case GENPOS_TRACK_CONFIG:
	case GENPOS_TRACK_SPEED:
		rmp440DataStrId->trackStr.trackMode = params->mode;
		break;
	default:
		*report = S_rmp440_BAD_REF;
		retval = ERROR;
	}
	return retval;
}


/*------------------------------------------------------------------------
 * GotoConfigCtrl  -  control codel of CONTROL request GotoConfig
 *
 * Description:
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
GotoCtrl(GENPOS_CART_REF *ref, int *report)
{
	GENPOS_CART_REF *refId = &SDI_F->ref;
	RMP440_STATUS_STR *status = &SDI_F->status;
	int retval = OK;

	/* Check if mode changed */
	switch (status->rs_mode) {
	case RMP440_POWER_CORD:
		*report = S_rmp440_POWER_CORD_CONNECTED;
		return ERROR;
	case RMP440_EMERGENCY:
		*report = S_rmp440_EMERGENCY_STOP;
		return ERROR;
	case RMP440_MOTORS_OFF:
		*report = S_rmp440_MOTORS_OFF;
		return ERROR;
	}
	/* Start correct tracking mode */
	*report = 0;
	switch (ref->dataType) {
	case GENPOS_POS_AND_SPEED_DATA:
		status->rs_mode = RMP440_TRACK;
		rmp440DataStrId->trackStr.trackMode = GENPOS_TRACK_CONFIG;
		break;
	case GENPOS_SPEED_DATA:
		status->rs_mode = RMP440_TRACK;
		rmp440DataStrId->trackStr.trackMode = GENPOS_TRACK_SPEED;
		break;
	default:
		*report = S_rmp440_BAD_REF;
		return ERROR;
	}
	memcpy(refId, ref, sizeof(GENPOS_CART_REF));
	return OK;
}

/*------------------------------------------------------------------------
 * Recale  -  control codel of CONTROL request SetPos
 *
 * Description:
 * Report (bilan): OK
 *                 S_rmp440_TRACKING_ON
 *
 * Return:    OK ou ERROR
 */

STATUS
Recale (RMP440_CART_3DCONFIG *setPosStr, int *bilan)
{

	GENPOS_CART_STATE *configId;
	RMP440_CART_3DSTATE *config3dId;

	configId = &rmp440DataStrId->robot;
	config3dId = &rmp440DataStrId->robot3d;
	
	configId->xRob = setPosStr->x;
	configId->yRob = setPosStr->y;
	config3dId->zRob = setPosStr->z;

	configId->theta = setPosStr->theta;

	SDI_F->gyro.gyroToRobotOffset = configId->theta	- SDI_F->gyro.gyroTheta;

	return OK;
}

#ifdef WITH_FELIB
/*------------------------------------------------------------------------
 * resCtrl  -  control codel of EXEC request ResetEmergencyStop
 *
 * Description:    
 * Report: OK
 *                 S_rmp440_EMERGENCY_STOP
 *
 * Returns:    OK or ERROR
 */

STATUS
resCtrl(int *report)
{
	RMP440_STATUS_STR *status = &SDI_F->status;

	if (status->rs_mode != RMP440_EMERGENCY) {
		*report = S_rmp440_NOT_EMERGENCY;
		return ERROR;
	}
	return OK;
}
#endif
